@@ -35,7 +35,6 @@ import (
 	"knative.dev/pkg/metrics/metricstest"
 	_ "knative.dev/pkg/metrics/testing"
 	rtesting "knative.dev/pkg/reconciler/testing"
-	"knative.dev/serving/pkg/activator/util"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
 	asmetrics "knative.dev/serving/pkg/autoscaler/metrics"
 	fakeservingclient "knative.dev/serving/pkg/client/injection/client/fake"
@@ -57,6 +56,7 @@ var (
 )
 
 type reqOp struct {
+	id   int
 	op   int
 	time int
 	key  types.NamespacedName
@@ -277,18 +277,22 @@ func TestStats(t *testing.T) {
 	}, {
 		name: "interleaved requests",
 		ops: []reqOp{{
+			id:   1,
 			op:   requestOpStart,
 			key:  rev1,
 			time: 0,
 		}, {
+			id:   2,
 			op:   requestOpStart,
 			key:  rev1,
 			time: 0,
 		}, {
+			id:   2,
 			op:   requestOpEnd,
 			key:  rev1,
 			time: 1,
 		}, {
+			id:   1,
 			op:   requestOpEnd,
 			key:  rev1,
 			time: 2,
@@ -317,14 +321,19 @@ func TestStats(t *testing.T) {
 			cr, _, cancel := newTestReporter(t)
 			defer cancel()
 
+			// statCache simulates the stat being kept for the entire request and reused
+			// for the request being finished.
+			statCache := map[int]*revisionStats{}
+
 			// Apply request operations
 			for _, op := range tc.ops {
 				time := time.Time{}.Add(time.Duration(op.time) * time.Millisecond)
 				switch op.op {
 				case requestOpStart:
-					cr.handleEvent(network.ReqEvent{Key: op.key, Type: network.ReqIn, Time: time})
+					statCache[op.id] = cr.handleRequestIn(network.ReqEvent{Key: op.key, Type: network.ReqIn, Time: time})
 				case requestOpEnd:
-					cr.handleEvent(network.ReqEvent{Key: op.key, Type: network.ReqOut, Time: time})
+					cr.handleRequestOut(statCache[op.id], network.ReqEvent{Key: op.key, Type: network.ReqOut, Time: time})
+					delete(statCache, op.id)
 				case requestOpTick:
 					stats := cr.report(time)
 					if len(stats) > 0 {
@@ -364,7 +373,7 @@ func TestStats(t *testing.T) {
 			})
 
 			if got, want := stats, tc.expectedStats; !cmp.Equal(got, want) {
-				t.Errorf("Unexpected stats (-want +got): %s", cmp.Diff(want, got))
+				t.Error("Unexpected stats (-want +got):", cmp.Diff(want, got))
 			}
 		})
 	}
@@ -381,9 +390,9 @@ func TestConcurrencyReporterRun(t *testing.T) {
 	}()
 
 	now := time.Time{}
-	cr.handleEvent(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
-	cr.handleEvent(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
-	cr.handleEvent(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
+	cr.handleRequestIn(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
+	cr.handleRequestIn(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
+	cr.handleRequestIn(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
 
 	reportCh <- now.Add(1)
 
@@ -409,7 +418,7 @@ func TestConcurrencyReporterRun(t *testing.T) {
 	got = append(got, <-cr.statCh...) // Scale from 0.
 	got = append(got, <-cr.statCh...) // Actual report.
 	if !cmp.Equal(got, want) {
-		t.Errorf("Unexpected stats (-want +got): %s", cmp.Diff(want, got))
+		t.Error("Unexpected stats (-want +got):", cmp.Diff(want, got))
 	}
 }
 
@@ -428,7 +437,7 @@ func TestConcurrencyReporterHandler(t *testing.T) {
 
 	resp := httptest.NewRecorder()
 	req := httptest.NewRequest(http.MethodPost, "http://example.com", nil)
-	rCtx := util.WithRevID(context.Background(), rev1)
+	rCtx := WithRevisionAndID(context.Background(), nil, rev1)
 
 	// Send a few requests.
 	handler.ServeHTTP(resp, req.WithContext(rCtx))
@@ -457,7 +466,74 @@ func TestConcurrencyReporterHandler(t *testing.T) {
 	got = append(got, <-cr.statCh...) // Scale from 0.
 	got = append(got, <-cr.statCh...) // Actual report.
 	if !cmp.Equal(got, want) {
-		t.Errorf("Unexpected stats (-want +got): %s", cmp.Diff(want, got))
+		t.Error("Unexpected stats (-want +got):", cmp.Diff(want, got))
+	}
+}
+
+func TestConcurrencyReporterRace(t *testing.T) {
+	cr, _, cancel := newTestReporter(t)
+	defer cancel()
+
+	base := time.Now()
+
+	// 1. Request in: Creates the stat (first part of handleRequestIn)
+	eventIn := network.ReqEvent{
+		Time: base,
+		Type: network.ReqIn,
+		Key:  rev1,
+	}
+	stats, _ := cr.getOrCreateStat(eventIn)
+
+	// 2. Report (will remove the stat if not guarded correctly)
+	cr.report(base.Add(1 * time.Second))
+
+	// 3. HandleEvent (second part of handleEvent)
+	stats.stats.HandleEvent(eventIn)
+
+	// 4. Request out: Creates a new stat, now negative
+	cr.handleRequestOut(stats, network.ReqEvent{
+		Time: base.Add(2 * time.Second),
+		Type: network.ReqOut,
+		Key:  rev1,
+	})
+
+	// 5. A negative report (if buggy)
+	got := cr.report(base.Add(2 * time.Second))
+	want := []asmetrics.StatMessage{{
+		Key: rev1,
+		Stat: asmetrics.Stat{
+			AverageConcurrentRequests: 1,
+			RequestCount:              1,
+			PodName:                   activatorPodName,
+		},
+	}}
+	if !cmp.Equal(got, want) {
+		t.Error("Unexpected stats (-want +got):", cmp.Diff(want, got))
+	}
+
+	// 6. It continues to work correctly.
+	stats = cr.handleRequestIn(network.ReqEvent{
+		Time: base.Add(2500 * time.Millisecond),
+		Type: network.ReqIn,
+		Key:  rev1,
+	})
+	cr.handleRequestOut(stats, network.ReqEvent{
+		Time: base.Add(3 * time.Second),
+		Type: network.ReqOut,
+		Key:  rev1,
+	})
+
+	got = cr.report(base.Add(3 * time.Second))
+	want = []asmetrics.StatMessage{{
+		Key: rev1,
+		Stat: asmetrics.Stat{
+			AverageConcurrentRequests: 0.5,
+			RequestCount:              1,
+			PodName:                   activatorPodName,
+		},
+	}}
+	if !cmp.Equal(got, want) {
+		t.Error("Unexpected stats (-want +got):", cmp.Diff(want, got))
 	}
 }
 
@@ -473,10 +549,10 @@ func TestMetricsReported(t *testing.T) {
 	}()
 
 	now := time.Time{}
-	cr.handleEvent(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
-	cr.handleEvent(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
-	cr.handleEvent(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
-	cr.handleEvent(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
+	cr.handleRequestIn(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
+	cr.handleRequestIn(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
+	cr.handleRequestIn(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
+	cr.handleRequestIn(network.ReqEvent{Key: rev1, Type: network.ReqIn, Time: now})
 
 	// Report
 	reportCh <- now.Add(1)
@@ -568,7 +644,7 @@ func BenchmarkConcurrencyReporterHandler(b *testing.B) {
 			Name:      testRevName + strconv.Itoa(i),
 		}
 		req := httptest.NewRequest(http.MethodGet, "http://example.com", nil)
-		reqs = append(reqs, req.WithContext(util.WithRevID(context.Background(), key)))
+		reqs = append(reqs, req.WithContext(WithRevisionAndID(context.Background(), nil, key)))
 
 		// Create revisions in the fake clients to trigger report logic.
 		rev := revision(key.Namespace, key.Name)
@@ -601,7 +677,7 @@ func BenchmarkConcurrencyReporterHandler(b *testing.B) {
 // requests for the respective time too.
 func BenchmarkConcurrencyReporterReport(b *testing.B) {
 	for _, revs := range []int{1, 5, 10, 50, 100, 200} {
-		b.Run(fmt.Sprintf("revs-%d", revs), func(b *testing.B) {
+		b.Run(fmt.Sprint("revs-", revs), func(b *testing.B) {
 			ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(b)
 			defer cancel()
 
@@ -623,7 +699,7 @@ func BenchmarkConcurrencyReporterReport(b *testing.B) {
 				revisions.Informer().GetIndexer().Add(rev)
 
 				// Send a sample request for each revision to make sure it's reported.
-				cr.handleEvent(network.ReqEvent{
+				cr.handleRequestIn(network.ReqEvent{
 					Time: time.Now(),
 					Type: network.ReqIn,
 					Key:  key,
