@@ -21,14 +21,14 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	"knative.dev/pkg/kmeta"
-	av1alpha1 "knative.dev/serving/pkg/apis/autoscaling/v1alpha1"
+	autoscalingv1alpha1 "knative.dev/serving/pkg/apis/autoscaling/v1alpha1"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
 	"knative.dev/serving/pkg/reconciler/revision/resources/names"
 )
 
 // MakePA makes a Knative Pod Autoscaler resource from a revision.
-func MakePA(rev *v1.Revision) *av1alpha1.PodAutoscaler {
-	return &av1alpha1.PodAutoscaler{
+func MakePA(rev *v1.Revision) *autoscalingv1alpha1.PodAutoscaler {
+	return &autoscalingv1alpha1.PodAutoscaler{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:            names.PA(rev),
 			Namespace:       rev.Namespace,
@@ -36,7 +36,7 @@ func MakePA(rev *v1.Revision) *av1alpha1.PodAutoscaler {
 			Annotations:     makeAnnotations(rev),
 			OwnerReferences: []metav1.OwnerReference{*kmeta.NewControllerRef(rev)},
 		},
-		Spec: av1alpha1.PodAutoscalerSpec{
+		Spec: autoscalingv1alpha1.PodAutoscalerSpec{
 			ContainerConcurrency: rev.Spec.GetContainerConcurrency(),
 			ScaleTargetRef: corev1.ObjectReference{
 				APIVersion: "apps/v1",
@@ -44,26 +44,26 @@ func MakePA(rev *v1.Revision) *av1alpha1.PodAutoscaler {
 				Name:       names.Deployment(rev),
 			},
 			ProtocolType: rev.GetProtocol(),
-			Reachability: func() av1alpha1.ReachabilityType {
+			Reachability: func() autoscalingv1alpha1.ReachabilityType {
 				// If the Revision has failed to become Ready, then mark the PodAutoscaler as unreachable.
 				if rev.Status.GetCondition(v1.RevisionConditionReady).IsFalse() {
 					// Make sure that we don't do this when a newly failing revision is
 					// marked reachable by outside forces.
 					if !rev.IsReachable() {
-						return av1alpha1.ReachabilityUnreachable
+						return autoscalingv1alpha1.ReachabilityUnreachable
 					}
 				}
 
 				// We don't know the reachability if the revision has just been created
 				// or it is activating.
 				if rev.Status.GetCondition(v1.RevisionConditionActive).IsUnknown() {
-					return av1alpha1.ReachabilityUnknown
+					return autoscalingv1alpha1.ReachabilityUnknown
 				}
 
 				if rev.IsReachable() {
-					return av1alpha1.ReachabilityReachable
+					return autoscalingv1alpha1.ReachabilityReachable
 				}
-				return av1alpha1.ReachabilityUnreachable
+				return autoscalingv1alpha1.ReachabilityUnreachable
 			}(),
 		},
 	}
